import {DOMSerializer} from 'prosemirror-model'

export default (options) => {
  const customClasses = JSON.parse(options.customClasses || '{}')

  return {
    patchSchema (schemaSpec) {
      addCustomClassesSupport(customClasses, schemaSpec.nodes)
      addCustomClassesSupport(customClasses, schemaSpec.marks)
    },
    blocksMenu () {
      return {
        selectClass: customClassMenu(customClasses),
      }
    },
  }
}

function customClassMenu (customClasses) {
  function filterAllowedClasses (classes, nodeOrMark) {
    const result = []
    for (const className of classes) {
      const classSettings = customClasses[className]
      const allowOn = classSettings['allowOn'] || []
      if (allowOn.includes(nodeOrMark.type.name)) {
        result.push(className)
      }
    }
    return result
  }

  return function (view) {
    let classSelect = document.createElement('select')

    classSelect.addEventListener('change', e => {
      e.preventDefault()
      const cursor = view.state.selection.$cursor
      const currentNode = cursor.parent

      let selectedClasses = []

      // First option is default style
      if (classSelect.selectedIndex !== 0) {
        selectedClasses = Array.from(classSelect.selectedOptions)
          .map(option => option.value)
          .filter(className => className !== undefined)
      }

      const newClasses = filterAllowedClasses(selectedClasses, currentNode)
      const newAttrs = {...currentNode.attrs, classes: newClasses}
      view.dispatch(
        view.state.tr.setNodeMarkup(cursor.pos - cursor.parentOffset - 1, null, newAttrs, currentNode.marks),
      )
    })

    return {
      dom: classSelect,
      update () {
        classSelect.innerHTML = ''
        classSelect.style.display = 'none'

        const cursor = view.state.selection.$cursor
        if (!cursor) {
          return
        }

        const currentNode = cursor.parent
        const currentNodeClasses = currentNode.attrs.classes || []

        const defaultOption = document.createElement('option')
        defaultOption.value = 0
        defaultOption.text = 'Defaut'
        defaultOption.selected = currentNodeClasses.length === 0
        classSelect.add(defaultOption)

        for (const className of filterAllowedClasses(Object.keys(customClasses), currentNode)) {
          const classSettings = customClasses[className]
          classSelect.style.display = null
          const option = document.createElement('option')
          option.value = className
          option.text = classSettings['label'] || className
          option.selected = currentNodeClasses.includes(className)
          classSelect.add(option)
        }
      },
    }
  }
}

function addCustomClassesSupport (customClasses, specs) {
  for (const specName in specs) {
    const allowedClasses = Object.keys(customClasses).filter(className => {
      const classSettings = customClasses[className]
      const allowOn = classSettings['allowOn'] || []
      return allowOn.includes(specName)
    })

    if (allowedClasses.length === 0) {
      continue
    }

    const spec = specs[specName]

    spec.attrs = Object.assign(spec.attrs || {}, { 'classes': { default: [] } })

    if (spec.parseDOM !== undefined) {
      const tagRules = spec.parseDOM.filter(rule => rule.tag !== undefined)

      for (const rule of tagRules) {
        const baseGetAttrs = rule.getAttrs

        rule.getAttrs = (node) => {
          let attrs

          if (baseGetAttrs) {
            attrs = baseGetAttrs(node)
            if (attrs === false) {
              return false
            }
          } else {
            attrs = rule.attrs
          }

          attrs = attrs || {}

          const classList = Array.from(node.classList)
          attrs.classes = classList.filter(klass => allowedClasses.includes(klass))
          return attrs
        }
      }
    }

    const baseToDOM = spec.toDOM
    if (baseToDOM !== undefined) {
      spec.toDOM = (node) => {
        const {dom, contentDOM} = DOMSerializer.renderSpec(window.document, baseToDOM(node))
        for (const klass of node.attrs.classes || []) {
          dom.classList.add(klass)
        }
        return {dom, contentDOM}
      }
    }
  }
}
