import getLanguage from './godo-menus-language.mjs'
import {toggleMark, setBlockType} from 'prosemirror-commands'

class Menu {
  constructor (menu, editorView) {
    this.items = {}
    this.editorView = editorView
    this.name = menu.name

    this.el = document.createElement('div')
    this.el.className = `godo-${this.name}-menu`
    this.el.wrapper = document.createElement('div')
    this.el.wrapper.className = 'menuicons'
    for (const [key, item] of Object.entries(menu.items)) {
      const newItem = item(editorView)
      this.items[key] = newItem
      this.el.wrapper.appendChild(newItem.dom)
    }
    this.el.appendChild(this.el.wrapper)

    this.update(editorView, null)
  }

  update (view, lastState) {
    let state = view.state

    if (lastState && lastState.doc.eq(state.doc) && lastState.selection.eq(state.selection)) {
      return
    }

    if (this.name === 'marks') {
      this.updateMarks(view, state)
    }

    for (const item of Object.values(this.items)) {
      item.update()
    }
  }

  updateMarks (view, state) {
    let $sel = state.selection
    const $cursor = $sel.$cursor
    const ranges = $sel.ranges

    // if single cursor
    if ($cursor && $cursor.marks()) {
      this.el.classList.add('fade')

    // else = select range
    } else {
      const range = ranges[0]
      if (range && ranges.length === 1) {
        this.el.classList.remove('fade')
      }
    }

    // move marks nav above selection
    let {from, to} = $sel
    let start = view.coordsAtPos(from)
    let end = view.coordsAtPos(to)
    let left = Math.max((start.left + end.left) / 2, start.left + 3)
    this.el.style.transform = `translate(${left}px, ${end.bottom}px)`

    // if (view.hasFocus()) this.el.hidden = false;
    this.el.hidden = !view.hasFocus()
  }

  destroy () { this.el.remove() }
}

function blocks (components, schema) {
  let menu = {
    name: 'blocks',
    items: {},
  }

  for (const component of components) {
    if (component.blocksMenu) {
      Object.assign(menu.items, component.blocksMenu(schema))
    }
  }

  return menu
}

function marks (components, schema) {
  let menu = {
    name: 'marks',
    items: {},
  }

  for (const component of components) {
    if (component.marksMenu) {
      Object.assign(menu.items, component.marksMenu(schema))
    }
  }

  return menu
}

function menuIcon (id, run, getIsActive) {
  return (view) => {
    let menuicon = document.createElement('button')
    menuicon.className = 'menuicon menuicon-' + id
    menuicon.setAttribute('type', 'button')
    menuicon.title = getLanguage(id).text
    menuicon.textContent = getLanguage(id).icon
    menuicon.addEventListener('mouseup', e => {
      e.preventDefault()
      run(view.state, view.dispatch, view)
    })

    return {
      dom: menuicon,
      update () {
        const state = view.state
        if (getIsActive(state)) {
          menuicon.classList.add('active')
        } else {
          menuicon.classList.remove('active')
        }

        menuicon.disabled = !run(state, null, view)
      },
    }
  }
}

// Helper function to create menu icons
// id: defined in languageContent
function marksMenuButton ({type, id, run}) {
  run = run || toggleMark(type)
  return menuIcon(id, run, (state) => {
    const $cursor = state.selection.$cursor

    // if single cursor
    if ($cursor && $cursor.marks()) {
      return $cursor.marks().some(m => type.name === m.type.name)
    }

    // else = select range
    const ranges = state.selection.ranges
    const range = ranges[0]
    if (range && ranges.length === 1){
      return state.doc.rangeHasMark(range.$from.pos, range.$to.pos, type)
    }

    return false
  })
}

function blocksMenuButton ({type, id, run, active, options}) {
  run = run || setBlockType(type, options)
  return menuIcon(id, run, (state) => {
    const activeDefault = () => {
      let {$from, to} = state.selection
      return to <= $from.end() && $from.parent.hasMarkup(type, options)
    }
    return (active) ? active(state) : activeDefault()
  })
}

export { Menu, blocks, marks, marksMenuButton, blocksMenuButton }
