import {Node} from 'prosemirror-model'

import {loadSchema} from '../../src-js/utils.mjs'
import {parse, serialize} from '../helpers'
import base from '../../src-js/components/base'
import customClasses from '../../src-js/components/custom-classes'

test.each([
  ['<p class="punk"><a>oï</a></p>', ['punk'], []],
  ['<p><a class="punk">oï</a></p>', [], ['punk']],
  ['<p class="reggae"><a>oï</a></p>', [], []],
  ['<p><a class="reggae">oï</a></p>', [], []],
])('parse dom nodes', (source, expectedParagraphClasses, expectedLinkClasses) => {
  const options = {
    'customClasses': {
      'punk': { 'allowOn': ['paragraph', 'link'] },
    },
  }

  const dummyLinkComponent = {
    marks: {
      link: { parseDOM: [{'tag': 'a'}] },
    },
  }

  const schema = loadSchema([base(), dummyLinkComponent, customClasses(options)])
  const node = parse(schema, source)
  expect(node.toJSON()).toEqual({
    type: 'doc',
    content: [{
      type: 'paragraph',
      attrs: { classes: expectedParagraphClasses },
      content: [{
        type: 'text',
        text: 'oï',
        marks: [{
          type: 'link',
          attrs: { classes: expectedLinkClasses},
        }],
      }],
    }],
  })
})

test('serialize to dom', () => {
  const options = {
    'customClasses': {
      'punk': { 'allowOn': ['paragraph', 'link'] },
    },
  }

  const dummyLinkComponent = {
    marks: {
      link: {
        toDOM () { return ['a', 0] },
      },
    },
  }

  const schema = loadSchema([base(), dummyLinkComponent, customClasses(options)])
  const doc = Node.fromJSON(schema, {
    type: 'doc',
    content: [{
      type: 'paragraph',
      attrs: { classes: ['punk'] },
      content: [{
        type: 'text',
        text: 'oï',
        marks: [{
          type: 'link',
          attrs: { classes: ['punk'] },
        }],
      }],
    }],
  })

  expect(serialize(schema, doc)).toBe('<p class="punk"><a class="punk">oï</a></p>')
})

test.each([
  [{volume: 'fort'}, undefined],
  [undefined, () => ({ volume: 'fort'})],
  [{volume: 'bas'}, () => ({ volume: 'fort'})],
])('keep base attributes', (attrs, getAttrs) => {
  const options = {
    'customClasses': { 'punk': { 'allowOn': ['link'] } },
  }

  const dummyLinkComponent = {
    marks: {
      link: {
        attrs: {volume: ''},
        parseDOM: [{
          tag: 'a',
          attrs: attrs,
          getAttrs: getAttrs,
        }],
      },
    },
  }

  const schema = loadSchema([base(), dummyLinkComponent, customClasses(options)])
  let node = parse(schema, '<p><a class="punk">oï</a></p>')
  expect(node.toJSON()).toEqual({
    type: 'doc',
    content: [{
      type: 'paragraph',
      content: [{
        type: 'text',
        text: 'oï',
        marks: [{
          type: 'link',
          attrs: { classes: ['punk'], volume: 'fort' },
        }],
      }],
    }],
  })
})


test('forward match error from getAttributes', async () => {
  const options = {
    'customClasses': { 'punk': { 'allowOn': ['link'] } },
  }

  const dummyLinkComponent = {
    marks: {
      link: {
        parseDOM: [{
          tag: 'a',
          getAttrs: () => false,
        }],
      },
    },
  }

  const schema = loadSchema([base(), dummyLinkComponent, customClasses(options)])
  let node = parse(schema, '<p><a class="punk">oï</a></p>')
  expect(node.toJSON()).toEqual({
    type: 'doc',
    content: [{
      type: 'paragraph',
      content: [{
        type: 'text',
        text: 'oï',
      }],
    }],
  })
})

