import {EditorState, Selection, TextSelection} from 'prosemirror-state'
import {DOMParser, DOMSerializer} from 'prosemirror-model'
import {history} from 'prosemirror-history'

import {loadSchema} from '../src-js/utils'
import {buildKeymap} from '../src-js/utils.mjs'
import {blocks, marks} from '../src-js/godo-menus.mjs'

export function parse (schema, html) {
  const parser = DOMParser.fromSchema(schema)
  let div = document.createElement('div')
  div.innerHTML = html.trim()
  return parser.parse(div)
}

export function serialize (schema, fragment) {
  const div = document.createElement('div')
  const dom = DOMSerializer
    .fromSchema(schema)
    .serializeFragment(fragment)
  div.appendChild(dom)
  return div.innerHTML
}

export class TestState {
  constructor (components, initialHtml) {
    const schema = loadSchema(components)
    const doc = parse(schema, initialHtml || '')

    this.schema = schema
    this.keymap = buildKeymap(components, this.schema, null, {})
    this.marksMenu = {}
    this.blocksMenu = {}

    for (const [key, item] of Object.entries(marks(components, this.schema).items)) {
      this.marksMenu[key] = item(this)
    }

    for (const [key, item] of Object.entries(blocks(components, this.schema).items)) {
      this.blocksMenu[key] = item(this)
    }

    this.state = EditorState.create({
      schema,
      doc,
      plugins: [history()],
    })

    this.dispatch = (tr) => this.apply(tr)
    this.apply(this.state.tr.setSelection(Selection.atEnd(doc)))
  }

  get html () { return serialize(this.schema, this.state.doc) }

  apply (tr) { this.state = this.state.apply(tr) }

  type (text) { this.apply(this.state.tr.insertText(text)) }

  focus () {}

  resolveTextOffset (node, textOffset) {
    if (node.isText) {
      return textOffset + node.marks.length
    }

    // 1 for node start token, don't count document start
    let resolvedOffset = node.type === this.schema.nodes.doc ? 0 : 1

    for (let childId = 0; childId < node.childCount; ++childId) {
      const child = node.child(childId)
      const childText = child.textContent
      if (textOffset <= childText.length) {
        return resolvedOffset + this.resolveTextOffset(child, textOffset)
      }
      textOffset -= childText.length
      resolvedOffset += child.nodeSize
    }
  }

  selectTextOffset (start, end) {
    const transaction = this.state.tr
    const doc = transaction.doc
    let selectionStart = this.resolveTextOffset(doc, start)
    let selectionEnd = end !== undefined ? this.resolveTextOffset(doc, end) : undefined
    const newSelection = TextSelection.create(doc, selectionStart, selectionEnd)
    this.apply(transaction.setSelection(newSelection))
  }

  select (text) {
    const textOffset = this.state.doc.textContent.indexOf(text)
    if (textOffset === -1) throw new Error('Can\'t find given text in document')
    this.selectTextOffset(textOffset, textOffset + text.length)
  }

  cursorAfter (text) {
    const textOffset = this.state.doc.textContent.indexOf(text)
    if (textOffset === -1) throw new Error('Can\'t find given text in document')
    this.selectTextOffset(textOffset + text.length)
  }

  key (keyScheme) { this.keymap[keyScheme](this.state, tr => this.apply(tr)) }

  clickBlocksMenu (name) {
    const event = new Event('mouseup')
    this.blocksMenu[name].dom.dispatchEvent(event)
  }

  clickMarksMenu (name) {
    const event = new Event('mouseup')
    this.marksMenu[name].dom.dispatchEvent(event)
  }
}

